'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var videojs = _interopDefault(require('video.js'));
var merge = _interopDefault(require('deepmerge'));

var version = "0.2.9";

var events = { "LOAD": "Load", "START": "Start", "PROGRESS": "Progress", "STREAMING_PROGRESS": "StreamingProgress", "FIRSTQUARTILE": "FirstQuartile", "MIDPOINT": "Midpoint", "THIRDQUARTILE": "ThirdQuartile", "COMPLETE": "Complete", "PAUSE": "Pause", "RESUME": "Resume", "CLOSE": "Close", "ENDED": "Ended", "START_BUFFERING": "start-buffering", "RE_BUFFERING": "re-buffering", "PROGRESSMARK": "ProgressMark", "PLAYBACKERROR": "PlaybackError" };
var quartiles = [{ "name": "ProgressMark", "value": 10 }, { "name": "FirstQuartile", "value": 25 }, { "name": "Midpoint", "value": 50 }, { "name": "ThirdQuartile", "value": 75 }, { "name": "Complete", "value": 95 }, { "name": "Ended", "value": 99 }];

var HLS = 'application/x-mpegurl';
var DASH = 'application/dash+xml';
var SMOOTH_STREAMING = 'application/vnd.ms-sstr+xml';
var FAIRPLAY = 'com.apple.fps.1_0';
var PLAYREADY = 'com.microsoft.playready';

function drmDetect(player) {
  var drmType = null;
  var formatType = null;
  var browserDRM = player.supportedDRM;
  var src = player.currentSource();

  if (src) {
    formatType = src.type;

    if (src.protected) {
      if (formatType === DASH) {
        drmType = getDashDRM(player, browserDRM);
      } else if (formatType === HLS && videojs.browser.IS_SAFARI) {
        drmType = FAIRPLAY;
      } else if (formatType === SMOOTH_STREAMING) {
        drmType = PLAYREADY;
      }
    }
  }

  return { formatType: formatType, drmType: drmType };
}

function getDashDRM(player, browserDRM) {
  var src = player.currentSource();
  if (videojs.browser.IS_EDGE || videojs.browser.IE_VERSION > 0) {
    return PLAYREADY;
  } else if (src.keySystemOptions && src.keySystemOptions.length > 0) {
    for (var i = 0; src.keySystemOptions.length > i; i++) {
      if (browserDRM.includes(src.keySystemOptions[i].name)) {
        return src.keySystemOptions[i].name;
      }
    }
    return src.keySystemOptions[0].name;
  }
  return null;
}

function playbackData(player) {
  var resolution = getResolution(player);

  if (player.dash && player.dash.shakaPlayer) {
    var shakaStats = player.dash.shakaPlayer.getStats();
    return {
      bitrate: shakaStats.streamBandwidth,
      resolution: resolution || {
        width: shakaStats.width,
        height: shakaStats.height
      }
    };
  } else if (player.tech_ && player.tech_.hls) {
    var hls = player.tech_.hls;
    var hlsStats = hls.playlists && hls.playlists.media();
    return {
      bitrate: hlsStats.attributes.BANDWIDTH,
      resolution: resolution || hlsStats.attributes.RESOLUTION
    };
  }

  return {
    bitrate: null,
    resolution: resolution
  };
}

function getResolution(player) {
  if (!player.tech_ || !player.tech_.el_) {
    return null;
  }

  var videoTag = player.tech_.el_;
  return {
    width: videoTag.videoWidth,
    height: videoTag.videoHeight
  };
}

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var Play = function () {
  function Play(player) {
    classCallCheck(this, Play);

    this.player = player;
    this.firstPlay = false;
    this.loadStart = 0;
    this.loadEnd = 0;
    this.secondsToLoad = 0;

    player.on('dispose', this.reset.bind(this));
    player.on('loadstart', this.onLoadStart.bind(this));
    player.on('loadeddata', this.onLoadedData.bind(this));
    player.on('playing', this.onPlaying.bind(this));
  }

  Play.prototype.reset = function reset() {
    this.firstPlay = false;
    this.loadStart = 0;
    this.loadEnd = 0;
    this.secondsToLoad = 0;
  };

  Play.prototype.onLoadStart = function onLoadStart() {
    this.reset();
    this.loadStart = new Date();
  };

  Play.prototype.onLoadedData = function onLoadedData() {
    this.loadEnd = new Date();
    this.secondsToLoad = (this.loadEnd - this.loadStart) / 1000;
  };

  Play.prototype.onPlaying = function onPlaying() {
    if (!this.firstPlay) {
      this.firstPlay = true;
      this.player.trigger('start-buffering', { secondsToLoad: +this.secondsToLoad.toFixed(3) });
      // console.log(`firstplay - secondsToLoad: ${this.secondsToLoad.toFixed(3)}`);
    }
  };

  return Play;
}();

var Buffering = function () {
  function Buffering(player) {
    classCallCheck(this, Buffering);

    this.player = player;
    this.timer = null;
    this.scrubbing = false;
    this.bufferPosition = false;
    this.bufferStart = false;
    this.bufferEnd = false;
    this.bufferCount = 0;
    this.readyState = false;

    player.on('dispose', this.reset.bind(this));
    player.on('loadstart', this.reset.bind(this));
    player.on('ended', this.reset.bind(this));
    player.on('pause', this.onPause.bind(this));
    player.on('waiting', this.onPlayerWaiting.bind(this));
    player.on('seeking', this.onSeeking.bind(this));
    player.on('canplaythrough', this.onCanPlayThrough.bind(this));
  }

  Buffering.prototype.reset = function reset() {
    if (this.timer) {
      clearTimeout(this.timer);
    }
    this.scrubbing = false;
    this.bufferPosition = false;
    this.bufferStart = false;
    this.bufferEnd = false;
    this.bufferCount = 0;
    this.readyState = false;
  };

  Buffering.prototype.onPause = function onPause() {
    this.bufferStart = false;
    this.scrubbing = this.player.scrubbing();
  };

  Buffering.prototype.onPlayerWaiting = function onPlayerWaiting() {
    if (this.bufferStart === false && this.player.currentTime() > 0) {
      this.bufferStart = new Date();
      this.bufferPosition = +this.player.currentTime().toFixed(0);
      this.readyState = +this.player.readyState();
    }
  };

  Buffering.prototype.onSeeking = function onSeeking() {
    this.scrubbing = true;
  };

  Buffering.prototype.onCanPlayThrough = function onCanPlayThrough() {
    var curTime = +this.player.currentTime().toFixed(0);
    var buffered = this.player.tech_ && this.player.tech_.el_ && this.player.tech_.el_.buffered;

    if (buffered && buffered.length > 0) {
      this.bufferPosition = +buffered.end(0).toFixed(0);
    }

    if (this.bufferStart && curTime !== this.bufferPosition) {
      this.bufferEnd = new Date();

      var secondsToLoad = (this.bufferEnd - this.bufferStart) / 1000;

      this.bufferStart = false;
      this.bufferPosition = false;
      this.bufferCount++;

      this.player.trigger('buffered', {
        currentTime: curTime,
        readyState: this.readyState,
        secondsToLoad: +secondsToLoad.toFixed(3),
        bufferCount: this.bufferCount,
        scrubbing: this.scrubbing
      });

      // console.log(`buffered (${this.bufferCount}) [${this.scrubbing}] - ${secondsToLoad.toFixed(3)}`);
    }

    if (this.bufferStart === false) {
      this.scrubbing = false;
    }
  };

  return Buffering;
}();

var STREAMING_PROGRESS_TIMER = 300000; // 300000 in ms = 5 min
var QUARTILE_CONFIG = {
  ALWAYS: 0,
  NO_SKIP: 1,
  ONLY_ONCE: 2
};

var noSkipNetworks = ["hbo"];

// Default options for the plugin.
var defaults$$1 = {
  url: "http://localhost:8889",
  contentId: "content1234",
  profileId: "prof1234",
  request: {
    headers: {
      Authorization: "JWT 1234"
    }
  }
};

var TrackEvents = function () {
  /**
   * Creates an instance of TrackEvents.
   * @param {videojs} player
   * @param {Object} options
   * @memberof TrackEvents
   */
  function TrackEvents(player, options) {
    classCallCheck(this, TrackEvents);

    // Player instance
    this.player = player;
    // Merged with default options
    this.options = options;
    // Saves last time
    this.lastTime = 0;
    // Playback Start Date
    this.startDate = Date.now();
    // Array with events already sent
    this.eventsSent = [];
    // Event number - sequence
    this.eventNumber = 1;
    // Current Source
    this.currentSrc = player.currentSource();
    // Seeking boolean
    this.seeking = false;
    // Quartile config always, noSkip, OnlyOnce [default: Always]
    this.quartileConfig = QUARTILE_CONFIG.ALWAYS;
    // If the content was paused and not resumed
    this.paused = false;
    // first play
    this.isFirstPlay = true;

    this.init();
  }

  /**
   * Function to invoke when the player is ready.
   *
   * @memberof TrackEvents
   */


  TrackEvents.prototype.onLoadedMetadata = function onLoadedMetadata() {
    this.currentSrc = this.player.currentSource();

    if (this.currentSrc && noSkipNetworks.includes(this.currentSrc.network)) {
      this.quartileConfig = QUARTILE_CONFIG.NO_SKIP;
    }

    this.sendEvent(events.START);

    this.intervalID = setInterval(this.sendEvent.bind(this, events.STREAMING_PROGRESS), STREAMING_PROGRESS_TIMER);
  };

  TrackEvents.prototype.onBuffered = function onBuffered(e, data) {
    this.sendEvent(events.RE_BUFFERING, data);
  };

  TrackEvents.prototype.onError = function onError(e, data) {
    var error = data ? data : this.player.error();
    this.sendEvent(events.PLAYBACKERROR, error.message);
  };

  TrackEvents.prototype.onFirstPlay = function onFirstPlay(e, data) {
    this.sendEvent(events.START_BUFFERING, data);
  };

  /**
   * Function to invoke when the event pause is triggered.
   *
   * @memberof TrackEvents
   */


  TrackEvents.prototype.onPauseEvent = function onPauseEvent() {
    if (this.player.currentTime() === this.player.duration()) {
      return;
    } else if (this.player.seeking()) {
      this.seeking = true;
      return;
    }

    this.sendEvent(events.PAUSE);
    this.paused = true;
  };

  /**
   * Function to invoke when the event play is triggered.
   *
   * @memberof TrackEvents
   */


  TrackEvents.prototype.onResumeEvent = function onResumeEvent() {
    if (this.isFirstPlay && this.player.startTime) {
      this.isFirstPlay = false;
    } else if (this.seeking) {
      this.seeking = false;
      return;
    } else if (!this.paused) {
      return;
    }

    this.sendEvent(events.RESUME);
    // Content was resumed
    this.paused = false;
  };

  /**
   * Function to invoke when the event ended is triggered.
   *
   * @memberof TrackEvents
   */


  TrackEvents.prototype.onEndedEvent = function onEndedEvent() {
    this.clearInterval(this.intervalID);
  };

  /**
   * Function to invoke when the event dispose is triggered.
   *
   * @memberof TrackEvents
   */


  TrackEvents.prototype.onDisposeEvent = function onDisposeEvent() {
    this.clearInterval(this.intervalID);
  };

  /**
   * Function to invoke when the event onbeforeunload is triggered.
   *
   * @memberof TrackEvents
   */


  TrackEvents.prototype.onBeforeUnload = function onBeforeUnload(event) {
    if (!navigator || !navigator.sendBeacon) {
      this.sendEvent(events.CLOSE);
    }

    var types = drmDetect(this.player);
    var pbData = playbackData(this.player);
    var url = this.options.url.includes('?') ? this.options.url + "&beacon=true" : this.options.url + "?beacon=true";
    var jwt = this.options.request && this.options.request.headers && this.options.request.headers.Authorization;

    navigator.sendBeacon(url, JSON.stringify({
      content: {
        id: this.options.contentId,
        drmType: types.drmType,
        formatType: types.formatType,
        playbackUrl: this.player.currentSrc && this.player.currentSrc()
      },
      events: this.getEventObject(events.CLOSE, null),
      playback: {
        position: Math.round(this.player.currentTime()),
        timeSpent: Math.round((Date.now() - this.startDate) / 1000),
        bitrate: pbData.bitrate,
        resolution: pbData.resolution
      },
      user: {
        profileId: this.options.profileId
      },
      playerID: this.player.playerID || this.player._playerID || '',
      jwt: jwt
    }));
  };

  /**
   * Function to invoke when the event timeupdate is triggered.
   *
   * @memberof TrackEvents
   */


  TrackEvents.prototype.onTimeUpdate = function onTimeUpdate() {
    var data = void 0;

    if (this.currentSrc && this.currentSrc.isBroadcast) {
      return;
    }

    var percentage = Math.round(this.player.currentTime() / this.player.duration() * 100);

    if (this.lastTime > this.player.currentTime()) {
      // Clean quartile events sent
      this.cleanEventsSent(percentage);
    }

    // Check quartiles
    //const event = getQuartileEvent(percentage);0
    var event = this.getAllQuartileEvents(percentage);

    if (event && event.length > 0) {
      this.sendEvent(event, data);
    }

    // Set lastTime with currentTime for quartile reset
    this.lastTime = this.player.currentTime();
  };

  /**
   *
   *
   * @param {any} id
   * @memberof TrackEvents
   */


  TrackEvents.prototype.clearInterval = function (_clearInterval) {
    function clearInterval(_x) {
      return _clearInterval.apply(this, arguments);
    }

    clearInterval.toString = function () {
      return _clearInterval.toString();
    };

    return clearInterval;
  }(function (id) {
    if (id) {
      clearInterval(id);
    }
  });

  /**
   * Sends event with player data
   *
   * @param {String} event
   * @memberof TrackEvents
   */


  TrackEvents.prototype.sendEvent = function sendEvent(event, data) {
    var types = drmDetect(this.player);
    var pbData = playbackData(this.player);

    var playerData = {
      content: {
        id: this.options.contentId,
        drmType: types.drmType,
        formatType: types.formatType,
        playbackUrl: this.player.currentSrc && this.player.currentSrc()
      },
      events: this.getEventObject(event, data),
      playback: {
        position: Math.round(this.player.currentTime()),
        timeSpent: Math.round((Date.now() - this.startDate) / 1000),
        bitrate: pbData.bitrate,
        resolution: pbData.resolution
      },
      user: {
        profileId: this.options.profileId
      },
      playerID: this.player.playerID || this.player._playerID || ''
    };

    if (data && (event === events.START_BUFFERING || event === events.RE_BUFFERING)) {
      playerData.bufferStats = data;
    }

    this.makeRequest(this.options.url, playerData, this.options.request);
  };

  /**
   * Returns event object with name & number of event
   *
   * @param {any} event
   * @returns
   * @memberof TrackEvents
   */


  TrackEvents.prototype.getEventObject = function getEventObject(event, data) {
    var _this = this;

    var events$$1 = [];

    if (Array.isArray(event)) {
      event.map(function (singleEvent) {
        var obj = {
          name: singleEvent.name || singleEvent,
          number: _this.eventNumber++
        };

        if (data && (singleEvent === events.PLAYBACKERROR || singleEvent === events.PROGRESSMARK)) {
          obj.value = data;
        } else if (singleEvent.value) {
          obj.value = singleEvent.value;
        }

        events$$1.push(obj);
      });
    } else {
      var obj = {
        name: event.name || event,
        number: this.eventNumber++
      };

      if (data && (event === events.PLAYBACKERROR || event === events.PROGRESSMARK)) {
        obj.value = data;
      } else if (event.value) {
        obj.value = event.value;
      }

      events$$1.push(obj);
    }
    return events$$1;
  };

  /**
   * Function to send request through videojs xhr.
   *
   * @param {any} url
   * @param {any} body
   * @param {any} request
   * @memberof TrackEvents
   */


  TrackEvents.prototype.makeRequest = function makeRequest(url, body, request) {
    var defRequest = {
      body: JSON.stringify(body),
      url: url,
      method: "POST",
      headers: {
        "Content-Type": "application/json"
      }
    };

    var req = merge(request, defRequest);
    videojs.xhr(req, function (err, res, body) {
      //idk
    });
  };

  /**
   * Returns quartile event if any
   *
   * @param {any} percentage
   * @returns
   * @memberof TrackEvents
   */


  TrackEvents.prototype.getQuartileEvent = function getQuartileEvent(percentage) {
    var _this2 = this;

    var filter = void 0;

    if (this.quartileConfig === QUARTILE_CONFIG.NO_SKIP) {
      filter = function filter(quartile) {
        return percentage === quartile.value && !_this2.eventsSent.includes(quartile.value);
      };
    } else {
      filter = function filter(quartile) {
        return percentage >= quartile.value && !_this2.eventsSent.includes(quartile.value);
      };
    }

    return quartiles.filter(filter);
  };

  /**
   * Returns all quartile events
   *
   * @param {any} percentage
   * @returns
   * @memberof TrackEvents
   */


  TrackEvents.prototype.getAllQuartileEvents = function getAllQuartileEvents(percentage) {
    var _this3 = this;

    var events$$1 = [];

    var quartiles$$1 = this.getQuartileEvent(percentage);
    if (quartiles$$1 && quartiles$$1.length > 0) {
      quartiles$$1.map(function (quartile) {
        events$$1.push(quartile);
        // Add event to array of events sent
        _this3.eventsSent.push(quartile.value);
      });
    }

    return events$$1;
  };

  /**
   * Clears events on rewind
   *
   * @param {any} percentage
   * @memberof TrackEvents
   */


  TrackEvents.prototype.cleanEventsSent = function cleanEventsSent(percentage) {
    this.eventsSent = this.eventsSent.filter(function (quartileNumber) {
      return percentage > quartileNumber;
    });
  };

  /**
   * Hooks player events
   *
   * @memberof TrackEvents
   */


  TrackEvents.prototype.hook = function hook(playerEvent, callback) {
    this.player.on(playerEvent, callback.bind(this));
  };

  /**
   * Init
   *
   * @memberof TrackEvents
   */


  TrackEvents.prototype.init = function init() {
    var _play = new Play(this.player);
    var _buffering = new Buffering(this.player);
    this.hook("loadedmetadata", this.onLoadedMetadata);
    this.hook("timeupdate", this.onTimeUpdate);
    this.hook("pause", this.onPauseEvent);
    this.hook("play", this.onResumeEvent);
    this.hook("ended", this.onEndedEvent);
    this.hook("dispose", this.onDisposeEvent);
    this.hook("buffered", this.onBuffered);
    this.hook("start-buffering", this.onFirstPlay);
    this.hook("error", this.onError);
    window.addEventListener("beforeunload", this.onBeforeUnload.bind(this));
  };

  return TrackEvents;
}();

// Cross-compatibility for Video.js 5 and 6.


var registerPlugin = videojs.registerPlugin || videojs.plugin;
var getPlugin = videojs.getPlugin || videojs.plugin;
// const dom = videojs.dom || videojs;

/**
 * A video.js plugin.
 *
 * In the plugin function, the value of `this` is a video.js `Player`
 * instance. You cannot rely on the player being in a "ready" state here,
 * depending on how the plugin is invoked. This may or may not be important
 * to you; if not, remove the wait for "ready"!
 *
 * @function trackingEvents
 * @param    {Object} [options={}]
 *           An object of options left to the plugin author to define.
 */
var trackingEvents = function trackingEvents(options) {
  var trackEvents = new TrackEvents(this, videojs.mergeOptions(defaults$$1, options));
};

// Register the plugin with video.js.
if (typeof getPlugin('eventTracking') === 'undefined') {
  registerPlugin("trackingEvents", trackingEvents);
}

// Include the version number.
trackingEvents.VERSION = version;

module.exports = trackingEvents;
